/*
* This file is part of Bielefeld Type Library (BTL).
*
* Copyright(c) Frederic Siepmann
* http://opensource.cit-ec.de/projects/btl
*
* This file may be licensed under the terms of of the
* GNU Lesser General Public License Version 3 (the ``LGPL''),
* or (at your option) any later version.
*
* Software distributed under the License is distributed
* on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
* express or implied. See the LGPL for the specific language
* governing rights and limitations.
*
* You should have received a copy of the LGPL along with this
* program. If not, go to http://www.gnu.org/licenses/lgpl.html
* or write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*
* The development of this software was supported by the
* Excellence Cluster EXC 277 Cognitive Interaction Technology.
* The Excellence Cluster EXC 277 is a grant of the Deutsche
* Forschungsgemeinschaft (DFG) in the context of the German
* Excellence Initiative.
*
* Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman Köster,
* 				Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling,
* 				Denis Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
*/

/*
 * PointCloudAnalysisData.cpp
 *
 *  Created on: Mar 21, 2011
 *      Author: lziegler
 */

#include "Cylinder.h"

using namespace airobots;
using namespace xmltio;
using namespace std;

const AngleUnit Cylinder::internalAngleUnit = RADIAN;
const LengthUnit Cylinder::internalLengthUnit = MILLIMETER;

Cylinder::Cylinder(const Point3D &position, const Rotation3D &orientation,
		double height, double radius, LengthUnit lengthUnit) :
	position(position), orientation(orientation), height(
			UnitConverter::convert(height, lengthUnit, internalLengthUnit)),
			radius(UnitConverter::convert(radius, lengthUnit,
					internalLengthUnit)) {
}

Cylinder::Cylinder() {
}

Cylinder::~Cylinder() {
}

string Cylinder::getBaseTag() {
	return "CYLINDER";
}

string Cylinder::getTemplate() {
	return "<" + getBaseTag() + "><GENERATOR/>" + Timestamp::getTemplate()
			+ "<POSITION x=\"\" y=\"\" z=\"\" />" + Rotation3D::getTemplate()
			+ "<HEIGHT /><RADIUS />" + "</" + getBaseTag() + ">";
}

void Cylinder::fillInto(Location &loc) const {

	Type::fillInto(loc);

	loc[XPath("RADIUS")] = getRadius(internalLengthUnit);
	loc[XPath("HEIGHT")] = getHeight(internalLengthUnit);
	loc[XPath("POSITION/@x")] = position.getX(internalLengthUnit);
	loc[XPath("POSITION/@y")] = position.getY(internalLengthUnit);
	loc[XPath("POSITION/@z")] = position.getZ(internalLengthUnit);

	Location rotloc = loc[XPath(Rotation3D::getBaseTag())];
	orientation.fillInto(rotloc);
}

void Cylinder::fromLocation(const Location &loc, Cylinder &object) {

	Type::fromLocation(loc, object);

	double radius = extract<double> (loc[XPath("RADIUS")]);
	double height = extract<double> (loc[XPath("HEIGHT")]);
	double px = extract<double> (loc[XPath("POSITION/@x")]);
	double py = extract<double> (loc[XPath("POSITION/@y")]);
	double pz = extract<double> (loc[XPath("POSITION/@z")]);

	Rotation3D rot;
	Location rotloc = loc[XPath(Rotation3D::getBaseTag())];
	Rotation3D::fromLocation(rotloc, rot);

	object.setRadius(radius, internalLengthUnit);
	object.setHeight(height, internalLengthUnit);
	object.setPosition(Point3D(px, py, pz, internalLengthUnit));
	object.setOrientation(rot);
}

void Cylinder::sanitizeLocation(Location &loc) const {
	XPath xp(getBaseTag());
	while (XPathIterator i = xp.evaluate(loc)) {
		Location l(*i);
		loc.remove(l);
	}
}

bool Cylinder::operator==(const Cylinder &other) const {
	return Type::operator ==(other) && other.radius == this->radius
			&& other.position == this->position && other.orientation
			== this->orientation && other.height == this->height;
}

Point3D Cylinder::getPosition() const {
	return position;
}
Rotation3D Cylinder::getOrientation() const {
	return orientation;
}
double Cylinder::getHeight(LengthUnit unit) const {
	return UnitConverter::convert(height, internalLengthUnit, unit);
}
double Cylinder::getRadius(LengthUnit unit) const {
	return UnitConverter::convert(radius, internalLengthUnit, unit);
}
void Cylinder::setPosition(const Point3D &position) {
	this->position = position;
}
void Cylinder::setOrientation(const Rotation3D &orientation) {
	this->orientation = orientation;
}
void Cylinder::setRadius(double radius, LengthUnit unit) {
	this->radius = UnitConverter::convert(radius, unit, internalLengthUnit);
}
void Cylinder::setHeight(double height, LengthUnit unit) {
	this->height = UnitConverter::convert(height, unit, internalLengthUnit);
}
