/*
* This file is part of Bielefeld Type Library (BTL).
*
* Copyright(c) Frederic Siepmann
* http://opensource.cit-ec.de/projects/btl
*
* This file may be licensed under the terms of of the
* GNU Lesser General Public License Version 3 (the ``LGPL''),
* or (at your option) any later version.
*
* Software distributed under the License is distributed
* on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
* express or implied. See the LGPL for the specific language
* governing rights and limitations.
*
* You should have received a copy of the LGPL along with this
* program. If not, go to http://www.gnu.org/licenses/lgpl.html
* or write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*
* The development of this software was supported by the
* Excellence Cluster EXC 277 Cognitive Interaction Technology.
* The Excellence Cluster EXC 277 is a grant of the Deutsche
* Forschungsgemeinschaft (DFG) in the context of the German
* Excellence Initiative.
*
* Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman Köster,
* 				Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling,
* 				Denis Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
*/

/*
 * MicroTimestamp.cpp
 *
 *  Created on: Aug 6, 2009
 *      Author: jwienke
 */

#include "MicroTimestamp.h"

#include <cmath>
#include <exception>
#include <stdexcept>

using namespace std;

namespace airobots {

std::string MicroTimestamp::getBaseTag() {
	return "READTIME";
}

std::string MicroTimestamp::getTemplate() {
	return std::string("<") + getBaseTag()
			+ "><SECONDS value=\"0\"/><MICROSECONDS value=\"0\"/></"
			+ getBaseTag() + ">";
}

timeval MicroTimestamp::timevalFromDouble(double time) {

	timeval val;
	val.tv_sec = (time_t) floor(time);
	val.tv_usec = (suseconds_t) ((time - val.tv_sec) * 1000000.0);

	return val;

}

MicroTimestamp::MicroTimestamp() {
}

MicroTimestamp::MicroTimestamp(timeval readtime) :
	readtime(readtime) {
	if (readtime.tv_usec < 0 || readtime.tv_usec >= 1000000) {
		std::domain_error outOfRange("Microseconds must be >= 0 and < 1000000!");
		throw outOfRange;
	}
}

MicroTimestamp::MicroTimestamp(time_t seconds, suseconds_t microSeconds) {
	if (microSeconds < 0 || microSeconds >= 1000000) {
		std::domain_error outOfRange("Microseconds must be >= 0 and < 1000000!");
		throw outOfRange;
	}

	readtime.tv_sec = seconds;
	readtime.tv_usec = microSeconds;
}

MicroTimestamp::MicroTimestamp(double time) :
	readtime(timevalFromDouble(time)) {

}

timeval MicroTimestamp::getTimeval() const {
	return readtime;
}

time_t MicroTimestamp::getSeconds() const {
	return readtime.tv_sec;
}
suseconds_t MicroTimestamp::getMicroSeconds() const {
	return readtime.tv_usec;
}

void MicroTimestamp::set(timeval time) {
	this->readtime = time;
}

bool MicroTimestamp::operator==(const MicroTimestamp & other) const {
	return (readtime.tv_sec == other.readtime.tv_sec) && (readtime.tv_usec
			== other.readtime.tv_usec);
}

std::ostream& operator<<(std::ostream& o, MicroTimestamp const& timestamp) {
	o << "MicroTimestamp(seconds = " << timestamp.getSeconds()
			<< ", microseconds = " << timestamp.getMicroSeconds() << ")";
	return o;
}

}

namespace xmltio {

void convert(const airobots::MicroTimestamp& ld, Location& loc) {

	loc[XPath("SECONDS/@value")] = ld.getSeconds();
	loc[XPath("MICROSECONDS/@value")] = ld.getMicroSeconds();

}

template<>
airobots::MicroTimestamp xmltio::extract<airobots::MicroTimestamp>::get() const {

	timeval time;
	time.tv_sec = extract<time_t> (loc_[XPath("SECONDS/@value")]);
	time.tv_usec = extract<time_t> (loc_[XPath("MICROSECONDS/@value")]);

	return airobots::MicroTimestamp(time);

}

}
