/*
* This file is part of Bielefeld Type Library (BTL).
*
* Copyright(c) Frederic Siepmann
* http://opensource.cit-ec.de/projects/btl
*
* This file may be licensed under the terms of of the
* GNU Lesser General Public License Version 3 (the ``LGPL''),
* or (at your option) any later version.
*
* Software distributed under the License is distributed
* on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
* express or implied. See the LGPL for the specific language
* governing rights and limitations.
*
* You should have received a copy of the LGPL along with this
* program. If not, go to http://www.gnu.org/licenses/lgpl.html
* or write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*
* The development of this software was supported by the
* Excellence Cluster EXC 277 Cognitive Interaction Technology.
* The Excellence Cluster EXC 277 is a grant of the Deutsche
* Forschungsgemeinschaft (DFG) in the context of the German
* Excellence Initiative.
*
* Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman Köster,
* 				Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling,
* 				Denis Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
*/

/*
 * ObjectPositionDataTest.cpp
 *
 *  Created on: Apr 28, 2010
 *      Author: lziegler
 */

#include <boost/test/unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>

#include <iostream>

#include <sys/time.h>
#include <time.h>

#include "btl/ObjectPositionData.h"
#include "utils.h"

using namespace airobots;
using namespace std;
using namespace xmltio;

static const double TOLERANCE = 0.0001;

BOOST_AUTO_TEST_SUITE(ObjectPositionDataTest)

BOOST_AUTO_TEST_CASE(testSelfCompatibility) {

	try {

		vector<pair<double,double> > polygon;
		polygon.push_back(pair<double,double>(0,0));
		polygon.push_back(pair<double,double>(1,1));
		polygon.push_back(pair<double,double>(2,2));

		ObjectPositionData original;
		original.setClassLabel(string("smacks"));

		ObjectPositionData::Hypothesis originalHyp;
		originalHyp.setGenerator("test");
		originalHyp.setTimestamp(Timestamp(253543797,253543712));
		originalHyp.setReliability(0.1);
		originalHyp.setPolygon(polygon, ObjectPositionData::MILLIMETER);
		originalHyp.setCoordinateKind("absolute");
		originalHyp.setReference("world");
		original.addHypothesis(originalHyp);

		BOOST_REQUIRE_CLOSE(originalHyp.getPolygon(ObjectPositionData::MILLIMETER)[2].first, originalHyp.getPolygon(ObjectPositionData::CENTIMETER)[2].first * 10, TOLERANCE);
		BOOST_REQUIRE_CLOSE(originalHyp.getPolygon(ObjectPositionData::MILLIMETER)[2].first, originalHyp.getPolygon(ObjectPositionData::METER)[2].first * 1000, TOLERANCE);
		BOOST_REQUIRE_CLOSE(originalHyp.getPolygon(ObjectPositionData::MILLIMETER)[2].first, originalHyp.getPolygon(ObjectPositionData::MILLIMETER)[2].first, TOLERANCE);
		originalHyp.setPolygon(polygon, ObjectPositionData::MILLIMETER);
		BOOST_REQUIRE_CLOSE(originalHyp.getPolygon(ObjectPositionData::MILLIMETER)[2].first, 2.0, TOLERANCE);
		originalHyp.setPolygon(polygon, ObjectPositionData::METER);
		BOOST_REQUIRE_CLOSE(originalHyp.getPolygon(ObjectPositionData::MILLIMETER)[2].first, 2000.0, TOLERANCE);
		originalHyp.setPolygon(polygon, ObjectPositionData::CENTIMETER);
		BOOST_REQUIRE_CLOSE(originalHyp.getPolygon(ObjectPositionData::MILLIMETER)[2].first, 20.0, TOLERANCE);
		originalHyp.setPolygon(polygon, ObjectPositionData::MILLIMETER);

		Location originalLoc(ObjectPositionData::getTemplate(), ObjectPositionData::getBaseTag());
		originalLoc = original;

		cout << originalLoc.getDocumentText(true) << endl;

		ObjectPositionData parsed = extract<ObjectPositionData>(originalLoc);
		ObjectPositionData::Hypothesis parsedHyp = parsed.getPositionHypotheses()[0];

		BOOST_REQUIRE_EQUAL(parsedHyp.getGenerator(), originalHyp.getGenerator());
		BOOST_REQUIRE_EQUAL(parsedHyp.getTimestamp(), originalHyp.getTimestamp());
		BOOST_REQUIRE_CLOSE(parsedHyp.getReliability(), originalHyp.getReliability(), TOLERANCE);
		BOOST_REQUIRE_EQUAL(parsed.getClassLabel(), original.getClassLabel());
		BOOST_REQUIRE_EQUAL(parsedHyp.getReference(), originalHyp.getReference());
		BOOST_REQUIRE_EQUAL(parsedHyp.getCoordinateKind(), originalHyp.getCoordinateKind());
		BOOST_REQUIRE_EQUAL(originalHyp.getPolygon(ObjectPositionData::MILLIMETER).size(), parsedHyp.getPolygon(ObjectPositionData::MILLIMETER).size());
		BOOST_REQUIRE_CLOSE(originalHyp.getPolygon(ObjectPositionData::MILLIMETER)[0].first, parsedHyp.getPolygon(ObjectPositionData::MILLIMETER)[0].first, TOLERANCE);
		BOOST_REQUIRE_CLOSE(originalHyp.getPolygon(ObjectPositionData::MILLIMETER)[1].first, parsedHyp.getPolygon(ObjectPositionData::MILLIMETER)[1].first, TOLERANCE);
		BOOST_REQUIRE_CLOSE(originalHyp.getPolygon(ObjectPositionData::MILLIMETER)[2].first, parsedHyp.getPolygon(ObjectPositionData::MILLIMETER)[2].first, TOLERANCE);

		BOOST_REQUIRE(original == parsed);
		parsed.setClassLabel("bla");
		BOOST_REQUIRE(!(original == parsed));

	} catch (std::exception e) {
		BOOST_FAIL(string("Caught exception: ") + e.what());
	}

}

BOOST_AUTO_TEST_CASE(testFileCompatibility) {

	Location fileContents(readFile("ObjectPositionData.xml"), ObjectPositionData::getBaseTag());
	ObjectPositionData parsed = extract<ObjectPositionData>(fileContents);
	ObjectPositionData::Hypothesis parsedHyp = parsed.getPositionHypotheses()[0];

	BOOST_REQUIRE_EQUAL("BoostDetector", parsedHyp.getGenerator());
	BOOST_REQUIRE_EQUAL(253543797, parsedHyp.getTimestamp().getInserted());
	BOOST_REQUIRE_EQUAL(253543712, parsedHyp.getTimestamp().getUpdated());
	BOOST_REQUIRE_CLOSE(0.6, parsedHyp.getReliability(), TOLERANCE);
	BOOST_REQUIRE_EQUAL("smacks", parsed.getClassLabel());
	BOOST_REQUIRE_EQUAL("world", parsedHyp.getReference());
	BOOST_REQUIRE_EQUAL("absolute", parsedHyp.getCoordinateKind());
	BOOST_REQUIRE_EQUAL(3.0, parsedHyp.getPolygon(ObjectPositionData::MILLIMETER).size());
	BOOST_REQUIRE_CLOSE(0.0, parsedHyp.getPolygon(ObjectPositionData::MILLIMETER)[0].first, TOLERANCE);
	BOOST_REQUIRE_CLOSE(1.0, parsedHyp.getPolygon(ObjectPositionData::MILLIMETER)[1].first, TOLERANCE);
	BOOST_REQUIRE_CLOSE(2.0, parsedHyp.getPolygon(ObjectPositionData::MILLIMETER)[2].first, TOLERANCE);
}

BOOST_AUTO_TEST_SUITE_END()
