/*
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * Copyright(c) Frederic Siepmann http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the GNU Lesser General Public
 * License Version 3 (the ``LGPL''), or (at your option) any later version.
 * 
 * Software distributed under the License is distributed on an ``AS IS'' basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the LGPL for the
 * specific language governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this program. If not,
 * go to http://www.gnu.org/licenses/lgpl.html or write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA.
 * 
 * The development of this software was supported by the Excellence Cluster EXC
 * 277 Cognitive Interaction Technology. The Excellence Cluster EXC 277 is a
 * grant of the Deutsche Forschungsgemeinschaft (DFG) in the context of the
 * German Excellence Initiative.
 * 
 * Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman
 * Köster, Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling, Denis
 * Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
 */
package de.unibi.airobots.btl.data;

import static de.unibi.airobots.btl.common.ElementParser.*;
import nu.xom.Attribute;
import nu.xom.Document;
import nu.xom.Element;
import nu.xom.Elements;
import de.unibi.airobots.btl.data.asu.ProcessingStatus;
import de.unibi.airobots.btl.data.asu.UtteranceCategory;
import de.unibi.airobots.btl.data.asu.WordNode;

public class AsuData extends Type {

	private static final String ELEM_META_INFO = "metaInfo";

	private static final String ELEM_ASR_BEHAVIOR = "asrBehavior";

	private static final String ELEM_PROCESSING_STATUS = "processingStatus";

	private static final String ELEM_SEMANTIC_INFO = "semanticInfo";

	private static final String ELEM_CATEGORY = "category";

	private static final String ELEM_PLAIN_TEXT = "plainText";

	private static final String ELEM_CONTENT = "content";

	private static final String ELEM_FRAME = "frame";

	private static final String ATTR_FRAME_NAME = "val";

	private static final String ATTR_SLOT = "slot";

	private static final String ELEM_WORD = "name";

	private static final String ELEM_RATING = "rating";

	/* Meta-Information */
	private ProcessingStatus processingStatus;

	private String asrBehavior;

	/* Semantic Information */
	private String plainText;

	private UtteranceCategory category = UtteranceCategory.fragment;

	private float rating = 0f;

	private WordNode rootNode;

	public static String getBaseTag() {
		return "utterance";
	}

	public static void fromElement(Element element, AsuData asuData) {
		Type.fromElement(element, asuData);
		parseMeta(getFirstChildElement(element, ELEM_META_INFO), asuData);
		parseSemantic(getFirstChildElement(element, ELEM_SEMANTIC_INFO),
				asuData);
		Element rating = getFirstChildElement(element, ELEM_RATING);
		asuData.rating = Float.parseFloat(rating.getValue());
	}

	private static void parseMeta(Element metaElem, AsuData asuData) {
		Element status = getFirstChildElement(metaElem, ELEM_PROCESSING_STATUS);
		asuData.processingStatus = ProcessingStatus.valueOf(status.getValue());
		Element asrBehavior = getFirstChildElement(metaElem, ELEM_ASR_BEHAVIOR);
		asuData.asrBehavior = asrBehavior.getValue();
	}

	private static void parseSemantic(Element semanticElem, AsuData asuData) {
		Element plainText = getFirstChildElement(semanticElem, ELEM_PLAIN_TEXT);
		asuData.plainText = plainText.getValue();
		Element category = getFirstChildElement(semanticElem, ELEM_CATEGORY);
		asuData.category = UtteranceCategory.valueOf(category.getValue());
		parseContent(getFirstChildElement(semanticElem, ELEM_CONTENT), asuData);
	}

	private static void parseContent(Element contentElem, AsuData asuData) {
		try {
			Element rootFrameElem = getFirstChildElement(contentElem,
					ELEM_FRAME);
			asuData.rootNode = createNodeFromElement(rootFrameElem);
		} catch (IllegalArgumentException ex) {
			// no frame found
		}
	}

	private static WordNode createNodeFromElement(Element e) {
		String frameName = e.getAttributeValue(ATTR_FRAME_NAME);
		String word = getFirstChildElement(e, ELEM_WORD).getValue();
		String slotName = e.getAttributeValue(ATTR_SLOT);
		WordNode node = new WordNode(word, frameName);
		node.setSlotName(slotName);
		Elements childs = e.getChildElements(ELEM_FRAME);
		for (int i = 0; i < childs.size(); i++) {
			Element c = childs.get(i);
			WordNode childNode = createNodeFromElement(c);
			node.addLeave(childNode);
		}
		return node;
	}

	public AsuData() {
		setLegacyParsing(true);
	}

	public ProcessingStatus getProcessingStatus() {
		return processingStatus;
	}

	public String getAsrBehavior() {
		return asrBehavior;
	}

	public UtteranceCategory getCategory() {
		return category;
	}

	public String getPlainText() {
		return plainText;
	}

	public float getRating() {
		return rating;
	}

	public WordNode getRootNode() {
		return rootNode;
	}

	public void setAsrBehavior(String asrBehavior) {
		this.asrBehavior = asrBehavior;
	}

	public void setCategory(UtteranceCategory category) {
		this.category = category;
	}

	public void setPlainText(String plainText) {
		this.plainText = plainText;
	}

	public void setProcessingStatus(ProcessingStatus processingStatus) {
		this.processingStatus = processingStatus;
	}

	public void setRating(float rating) {
		this.rating = rating;
	}

	public void setRootNode(WordNode rootNode) {
		this.rootNode = rootNode;
	}

	/**
	 * Fills the given parent node with the contents of this type.
	 * 
	 * @param parent
	 *            parent node that will be modified to contain the information
	 *            stored in this type
	 */
	@Override
	public void fillInto(Element parent) {
		super.fillInto(parent);

		Element metaInfo = new Element(ELEM_META_INFO);
		Element semanticInfo = new Element(ELEM_SEMANTIC_INFO);

		/* asu.meta information: */
		Element timestampNode = new Element("timeStamp");
		timestampNode.appendChild(String.valueOf(System.currentTimeMillis()));
		Element statusNode = new Element(ELEM_PROCESSING_STATUS);
		final ProcessingStatus state = ProcessingStatus.forRating(this.rating);
		statusNode.appendChild(state.toString());
		Element asrBehavior = new Element(ELEM_ASR_BEHAVIOR);
		asrBehavior.appendChild(this.asrBehavior);
		metaInfo.appendChild(timestampNode);
		metaInfo.appendChild(statusNode);
		metaInfo.appendChild(asrBehavior);

		/* semantic information */
		Element plainTextNode = new Element(ELEM_PLAIN_TEXT);
		plainTextNode.appendChild(this.plainText);
		Element categoryNode = new Element(ELEM_CATEGORY);
		categoryNode.appendChild(this.category.name());
		Element contentElem = new Element(ELEM_CONTENT);
		if (this.rootNode != null) {
			contentElem.appendChild(wordNodeToElement(this.rootNode));
		}
		semanticInfo.appendChild(plainTextNode);
		semanticInfo.appendChild(categoryNode);
		semanticInfo.appendChild(contentElem);

		/* add rating */
		Element ratingNode = new Element(ELEM_RATING);
		ratingNode.appendChild(String.valueOf(this.rating));

		parent.appendChild(metaInfo);
		parent.appendChild(semanticInfo);
		parent.appendChild(ratingNode);
	}

	/**
	 * Returns this utterance as DOM document.
	 * 
	 * @return DOM document
	 */
	public final Document toDocument() {
		Element root = new Element(getBaseTag());
		this.fillInto(root);
		return new Document(root);
	}

	private Element wordNodeToElement(final WordNode wordNode) {
		assert wordNode != null : "got null node";
		Element frame = new Element(ELEM_FRAME);
		Attribute val = new Attribute(ATTR_FRAME_NAME, wordNode.getFrameName());
		frame.addAttribute(val);
		if (wordNode.getSlotName() != null) {
			String slotName = wordNode.getSlotName();
			Attribute slot = new Attribute(ATTR_SLOT, slotName);
			frame.addAttribute(slot);
		}
		Element name = new Element(ELEM_WORD);
		name.appendChild(wordNode.getWord());
		frame.appendChild(name);
		for (WordNode childNode : wordNode.getLeaves()) {
			frame.appendChild(wordNodeToElement(childNode));
		}
		return frame;
	}

}
