/*
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * Copyright(c) Frederic Siepmann http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the GNU Lesser General Public
 * License Version 3 (the ``LGPL''), or (at your option) any later version.
 * 
 * Software distributed under the License is distributed on an ``AS IS'' basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the LGPL for the
 * specific language governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this program. If not,
 * go to http://www.gnu.org/licenses/lgpl.html or write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA.
 * 
 * The development of this software was supported by the Excellence Cluster EXC
 * 277 Cognitive Interaction Technology. The Excellence Cluster EXC 277 is a
 * grant of the Deutsche Forschungsgemeinschaft (DFG) in the context of the
 * German Excellence Initiative.
 * 
 * Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman
 * Köster, Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling, Denis
 * Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
 */
package de.unibi.airobots.btl.data;

import nu.xom.Attribute;
import nu.xom.Element;
import nu.xom.ParsingException;

/**
 * This class represents a location in world Coordinates (X[m], Y[m], Yaw[rad]),
 * which is sent as goal to the robot. It is also possible to adjust the
 * tolerance of the short-range navigation using coordinateTolerance and
 * yawTolerance.
 * 
 * @author unknown
 * @author jwienke
 */
public class NavigationGoalData extends PositionData {
	/**
	 * The type of this goal that determines the method the robot uses to
	 * achieve reaching this goal.
	 */
	public static enum GoalType {
		/** Just use local obstacle avoidance. */
		LOCAL,
		/** Use a global navigation planner. */
		GLOBAL
	};

	private double coordinateTolerance = 0.25;
	private double yawTolerance = Math.PI * 2;
	private GoalType type = GoalType.LOCAL;

	/**
	 * Constructs a new navigation goal. This is either achieved using just
	 * local obstacle avoidance or using a global navigation planner, depending
	 * on the type.
	 * 
	 * @param generator
	 *            generator of this data
	 * @param x
	 *            The x coordinate of this goal. World coordinates [m].
	 * @param y
	 *            The y coordinate of this goal. World coordinates [m].
	 * @param yaw
	 *            The yaw (orientation) of this goal [rad].
	 * @param coordinateTolerance
	 *            How close we have to get to the goal to "reach" it [m].
	 * @param yawTolerance
	 *            How close we have to get to the target orientation to "reach"
	 *            it [rad].
	 * @param type
	 *            The method used to achieve reaching this goal (GLOBAL or
	 *            LOCAL).
	 * @see GoalType
	 */
	public NavigationGoalData(String generator, double x, double y, double yaw,
			double coordinateTolerance, double yawTolerance, GoalType type) {

		super(x, y, yaw, System.currentTimeMillis());

		setGenerator(generator);

		this.coordinateTolerance = coordinateTolerance;
		this.yawTolerance = yawTolerance;
		this.type = type;
	}

	public NavigationGoalData(final PositionData positionData) {
		setX(positionData.getX());
		setY(positionData.getY());
		setYaw(positionData.getYaw());
	}

	/**
	 * Constructs a new navigation goal. This is either achieved using just
	 * local obstacle avoidance or using a global navigation planner, depending
	 * on the type.
	 * 
	 * @param generator
	 *            the generator of the data
	 * @param x
	 *            The x coordinate of this goal. World coordinates [m].
	 * @param y
	 *            The y coordinate of this goal. World coordinates [m].
	 * @param yaw
	 *            The yaw (orientation) of this goal [rad].
	 * @param type
	 *            The method used to achieve reaching this goal (GLOBAL or
	 *            LOCAL).
	 * @see GoalType
	 */
	public NavigationGoalData(String generator, double x, double y, double yaw,
			GoalType type) {
		this(generator, x, y, yaw, 0.25, Math.PI * 2, type);
	}

	/**
	 * Constructor, sets X, Y and Yaw as 0.0, CoordinateTolerance as 0.1,
	 * yawTolerance as 2PI and the type as LOCAL.
	 */
	public NavigationGoalData() {
		this("javabtl", 0.0, 0.0, 0.0, GoalType.LOCAL);
	}

	/**
	 * @return the coordinateTolerance[m] of the goal.
	 */
	public double getCoordinateTolerance() {
		return coordinateTolerance;
	}

	/**
	 * @return the yawTolerance[rad] of the goal.
	 */
	public double getYawTolerance() {
		return yawTolerance;
	}

	/**
	 * Gets the type of this goal.
	 * 
	 * @return {@link GoalType#LOCAL} or {@link GoalType#LOCAL}, never null.
	 */
	public GoalType getType() {
		return type;
	}

	/**
	 * Changes the type ({@link GoalType#LOCAL} or {@link GoalType#LOCAL}) of
	 * this goal.
	 * 
	 * @param type
	 *            The {@link GoalType} of this goal, not null.
	 */
	public void setType(GoalType type) {
		if (type == null) {
			throw new NullPointerException(
					"The type of a goal should never be null!");
		}
		this.type = type;
	}

	/**
	 * Sets the tolerance of the Coordinates. Think of it as a circle with the
	 * given radius, which is "drawn" around the goal. If the center of the
	 * robot reaches the circle the goal is reached.
	 * 
	 * Normally used between 0.1 an 0.3.
	 * 
	 * @param coordinateTolerance
	 */
	public void setCoordinateTolerance(double coordinateTolerance) {
		this.coordinateTolerance = coordinateTolerance;
	}

	/**
	 * Set the yawTolerance[rad], in many cases the oriantation of the robot is
	 * not important. The tolerance should then be set to 2*PI. Otherwise values
	 * around PI*0.1 will work quiet well.
	 * 
	 * @param yawTolerance
	 */
	public void setYawTolerance(double yawTolerance) {
		this.yawTolerance = yawTolerance;
	}

	public static String getBaseTag() {
		return "NAVIGATIONGOAL";
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void fillInto(Element parent) {

		super.fillInto(parent);

		// <TOLERANCES>
		Element tolerances = new Element("TOLERANCES");
		parent.appendChild(tolerances);

		// <DR>
		Element dr = new Element("DR");
		Attribute attdr = new Attribute("quant",
				Double.toString(coordinateTolerance));
		dr.addAttribute(attdr);
		tolerances.appendChild(dr);

		// <DTHETA>
		Element dtheta = new Element("DTHETA");
		Attribute attdtheta = new Attribute("quant",
				Double.toString(yawTolerance));
		dtheta.addAttribute(attdtheta);
		tolerances.appendChild(dtheta);

		// <TYPE>
		Element typeElem = new Element("TYPE");
		typeElem.appendChild(type.name());

		// Append all child elements to <GOAL>
		parent.appendChild(typeElem);

	}

	public static void fromElement(Element element, NavigationGoalData type)
			throws ParsingException {

		PositionData.fromElement(element, type);

		double dr = 0.25;
		double dtheta = Math.PI * 2;

		GoalType goalType = GoalType.LOCAL;

		Element tolerances = (Element) element.query("TOLERANCES").get(0);
		Element drElement = (Element) tolerances.getChildElements("DR").get(0);
		dr = Double.parseDouble(drElement.getAttributeValue("quant"));
		Element dtElement = (Element) tolerances.getChildElements("DTHETA")
				.get(0);
		dtheta = Double.parseDouble(dtElement.getAttributeValue("quant"));

		Element typeElement = (Element) element.getChildElements("TYPE").get(0);
		String typeString = typeElement.getValue();
		if (typeString != null) {
			goalType = GoalType.valueOf(typeString);
		}

		type.setCoordinateTolerance(dr);
		type.setYawTolerance(dtheta);
		type.setType(goalType);

	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public boolean equals(Object obj) {

		if (!(obj instanceof NavigationGoalData)) {
			return false;
		}

		if (obj == this) {
			return true;
		}

		NavigationGoalData other = (NavigationGoalData) obj;

		return super.equals(obj)
				&& close(other.getCoordinateTolerance(),
						getCoordinateTolerance())
				&& close(other.getYawTolerance(), getYawTolerance())
				&& (other.getType().equals(getType()));

	}

	@Override
	public String toString() {
		return "NavigationGoalData[ x:" + getX() + " y:" + getY() + " yaw:" + getYaw()
				+ " coordinateTolerance:" + coordinateTolerance + " yawTolerance:" + yawTolerance
				+ " type:" + type.name() + " ]";
	}
}
